/*
    Copyright 2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "uologin.h"

#ifdef WIN32
    #include <winsock.h>
    #include "winerr.h"

    #define close   closesocket
    #define sleep   Sleep
#else
    #include <unistd.h>
    #include <sys/socket.h>
    #include <sys/types.h>
    #include <arpa/inet.h>
    #include <netinet/in.h>
    #include <netdb.h>
#endif



#define VER         "0.1"
#define BUFFSZ      2048
#define HOST        "login.owo.com"
#define PORT        7775



u_int resolv(char *host);
void std_err(void);



int main(int argc, char *argv[]) {
    struct  sockaddr_in peer;
    u_int   ip;
    int     sd,
            len;
    u_char  buff[BUFFSZ],
            *errmsg;
    char    *uoerror[] = {
            "Either the Account Name or Password you provided were incorrect.  If this is a new account your account may not be active yet.  Please try again shortly.",
            "Someone is already using this account.",
            "Your account has been blocked.",
            "Your account credentials are invalid. Check your user ID and password and try again.",
            NULL,
            "The IGR concurrency limit has been met.",
            "The IGR time limit has been met",
            "A general IGR authentication failure has occurred",
            NULL,
            "You cannot currently log into this shard because a character transfer is in progress."
            };

#ifdef WIN32
    WSADATA    wsadata;
    WSAStartup(MAKEWORD(1,0), &wsadata);
#endif


    setbuf(stdout, NULL);

    fputs("\n"
        "Ultima Online account checker "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 3) {
        printf("\n"
            "Usage: %s <username> <password>\n"
            "\n", argv[0]);
        exit(1);
    }

    fputs("- resolv " HOST "...\n", stdout);
    peer.sin_addr.s_addr = resolv(HOST);
    peer.sin_port        = htons(PORT);
    peer.sin_family      = AF_INET;

    sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if(sd < 0) std_err();

    printf("- connect to %s : %hu\n", inet_ntoa(peer.sin_addr), PORT);
    if(connect(sd, (struct sockaddr *)&peer, sizeof(peer)) < 0) {
        peer.sin_port    = htons(PORT + 1);
        printf("- connect to %s : %hu\n", inet_ntoa(peer.sin_addr), PORT + 1);
        if(connect(sd, (struct sockaddr *)&peer, sizeof(peer))
          < 0) std_err();
    }

    fputs("- generate fake source IP address and encryption key\n", stdout);
    ip = ~time(NULL);
    uocalcenc(ip);

    fputs("- send fake IP address\n", stdout);
    *(u_int *)buff = htonl(ip);
    if(send(sd, buff, 4, 0)
      < 0) std_err();

    fputs("- send username and password\n", stdout);
    len = uobuildlogin(buff, argv[1], argv[2]);
    if(send(sd, buff, len, 0)
      < 0) std_err();

    fputs("- receive data\n", stdout);
    recv(sd, buff, 1, 0);
    len = uopcklen(buff);

    if(len != 2) {
        fputs("\n  Your account IS valid!!!\n\n", stdout);
    } else {
        recv(sd, buff, 1, 0);
        if((*buff >= (sizeof(uoerror) / sizeof(char *))) || !uoerror[*buff]) {
            errmsg =
                "There is some problem communicating with Origin server.\n"
                "Please restart Ultima Online and try again.";
        } else {
            errmsg = uoerror[*buff];
        }
        printf("\n"
            "  Login failed: your account doesn't seem valid:\n"
            "\n"
            "%s\n"
            "\n", errmsg);
    }

    close(sd);
    return(0);
}



u_int resolv(char *host) {
    struct  hostent *hp;
    u_int   host_ip;

    host_ip = inet_addr(host);
    if(host_ip == INADDR_NONE) {
        hp = gethostbyname(host);
        if(!hp) {
            printf("\nError: Unable to resolv hostname (%s)\n", host);
            exit(1);
        } else host_ip = *(u_int *)(hp->h_addr);
    }
    return(host_ip);
}



#ifndef WIN32
    void std_err(void) {
        perror("\nError");
        exit(1);
    }
#endif


